%single_slit_general
%
% Computes the normalized wavefield power as a function of range r and
% polar angle theta from a finite width slit. Rather than assume Fresnel
% and Fraunhofer approximations, the wavefield integral is evaluated
% numerically.
%
% LAST UPDATED by Andy French November 2015

function single_slit_general

%Set wavelength /nm
%http://science-edu.larc.nasa.gov/EDDOCS/Wavelengths_for_Colors.html
lamda = 400;  %Violet
lamda = 445;  %Indigo
lamda = 475;  %Blue
lamda = 510;  %Green
lamda = 570;  %Yellow
lamda = 590;  %Orange
lamda = 650;  %Red

%Convert wavelength to m
lamda = lamda*1e-9;

%Define slit width /lamda
w = 10;

%Fraunhofer range /lamda
r_F = w.^2;

%Define max range /lamda
rmax = 2*r_F;

%Define min range /lamda
rmin = 0.05*rmax;

%Define maximum angle /radians
theta_max = pi/2;

%Fontsize
fsize = 18;

%Min dB value
dBmin = -50;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Set number of integration steps
N = 500;

%Set number of range and angle evaluations
R = 300;
A = 200;

%Define polar angles /radians
theta = linspace(-pi/2,pi/2,A);

%Define ranges /m
r = linspace(rmin,rmax,R);

%Vectorize
[r,theta] = meshgrid(r,theta);
rr = repmat(r,[1,1,N]);
tt = repmat(theta,[1,1,N]);

%Evaluate integral
zz = ones(A,R,N);
z = linspace(-w/2,w/2,N);
for n=1:N
    zz(:,:,n) = z(n);
end
dz = w/N;
p = sqrt( rr.^2 +zz.^2 -2*rr.*zz.*sin(tt) );
clear zz tt
phi = dz*sum( (rr./p).*exp( 2*pi*1i*p ), 3 );
clear p rr

%Define intensity
I = (abs(phi)).^2;
clear phi

%Plot surface
figure;
map = interp_colormap(N);
surf( r,180*theta/pi,dBm(I));
shading interp;
xlabel('Range /\lambda','fontsize',fsize );
ylabel('Polar angle \theta / ^o','fontsize',fsize);
title({'Diffraction pattern of a single slit x r^2',...
    ['\lambda = ',num2str(lamda*1e9),'nm, w = ',num2str(w),'\lambda']},'fontsize',fsize );
set(gca,'fontsize',fsize);
view(2);
xlim([0,rmax]);
ylim([-theta_max*180/pi,theta_max*180/pi])
zlim([dBmin,0])
caxis([dBmin,0])
colorbar('fontsize',fsize)
print( gcf,['generalized single slit rmax=',num2str(rmax),' w=',num2str(w),'.png'],'-r300','-dpng');
view([-50,60]);
print( gcf,['3D generalized single slit rmax=',num2str(rmax),' w=',num2str(w),'.png'],'-r300','-dpng');
close(gcf);

%Plot x,y,I surface
figure;
map = interp_colormap(N);
surf( r.*cos(theta),r.*sin(theta),dBm(I));
shading interp;
xlabel('x /\lambda','fontsize',fsize );
ylabel('y /\lambda','fontsize',fsize );
title({'Diffraction pattern of a single slit x r^2',...
    ['\lambda = ',num2str(lamda*1e9),'nm, w = ',num2str(w),'\lambda']},'fontsize',fsize );
set(gca,'fontsize',fsize);
axis vis3d
view(2);
xlim([0,rmax]);
ylim([-rmax,rmax]);
zlim([dBmin,0])
caxis([dBmin,0])
colorbar('fontsize',fsize)
print( gcf,['xy generalized single slit rmax=',num2str(rmax),' w=',num2str(w),'.png'],'-r300','-dpng');
view([-50,60]);
print( gcf,['3D xy generalized single slit rmax=',num2str(rmax),' w=',num2str(w),'.png'],'-r300','-dpng');
close(gcf);

%%

%Normalized dB function
function y = dBm(x)
y = NaN(size(x));
k = x>0;
y(k) = 10*log10(x(k));
y = y - max(reshape(y,[1,numel(y)]));

%%

%dB function
function y = dB(x)
y = NaN(size(x));
k = x>0;
y(k) = 10*log10(x(k));

%End of code